# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
$LOAD_PATH.unshift(File.absolute_path("#{__dir__}/../.."))
$LOAD_PATH.unshift(File.absolute_path("#{__dir__}/../../../../easy_type/lib"))
require 'easy_type'
require 'puppet_x/enterprisemodules/oracle/access'
require 'puppet_x/enterprisemodules/oracle/title_parser'
require 'puppet_x/enterprisemodules/oracle/default_sid'
require 'puppet_x/enterprisemodules/oracle/source_functions'
require 'puppet_x/enterprisemodules/oracle/java_source_handling'

Puppet::Type.newtype(:ora_java_source) do
  include EasyType
  extend PuppetX::EnterpriseModules::Oracle::TitleParser
  include PuppetX::EnterpriseModules::Oracle::Access
  include PuppetX::EnterpriseModules::Oracle::DefaultSid

  desc <<-DESC
  This resource allows you to manage PL/SQL procedures in the Oracle database.

      ora_java_source { 'testuser.my_java_class':
        ensure  => 'present',
        source => '/vagrant/tests/java_source.sql',
      }

  This puppet definition ensures that the procedure `testuser.my_java_class` is available
  in the database and that its content matches the content defined in the specified source.

  To decide if the procedure needs an update, the puppet type compares the content in the database, with
  the content in the source file. This comparison is done insenitive to case, white spacing and used quote's
  (the " or the ` ).

  DESC

  to_get_raw_resources do
    sql_on_all_primary_database_sids "select distinct owner, object_name as object_name from dba_objects where object_type = 'JAVA SOURCE'"
  end

  on_create do
    apply
  end

  on_modify do
    apply
  end

  on_destroy do
    [:sql, "drop java source \"#{owner}\".\"#{java_name}\"", { :sid => sid }]
  end

  def pre_run_check
    return if self[:ensure].to_s == 'absent'
    raise Puppet::Error, 'At least one source or content need to be set up' unless source || content
    raise Puppet::Error, 'Only one source or content can be set up' if source && content
  end

  def apply
    options = { :sid => sid, :parse => false, :catch_extra_errors => /compilation errors/ }
    options[:failonsqlfail] = report_errors

    statement = if content
                  "#{content}\n/\n"
                else
                  "@#{source}\n/\n"
                end
    if cwd
      fail "Working directory '#{cwd}' does not exist" unless File.directory?(cwd)

      FileUtils.cd(cwd)
    end
    output = sql statement, options
    Puppet.debug(output) if logoutput.to_s == 'true'
    nil
  end

  map_title_to_sid(:owner, :java_name) { /(.+)\.(.+)/ }

  ensurable

  parameter :name
  parameter :owner
  parameter :java_name
  parameter :sid
  parameter :timeout
  parameter :cwd
  parameter :logoutput
  parameter :report_errors

  property  :source
  property  :content
  ora_autorequire(:ora_user, :owner)
end
