# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#

# Defines a new property for the content of the Oracle Java source function.
newproperty(:content) do
  include EasyType
  include PuppetX::EnterpriseModules::Oracle::Access
  include PuppetX::EnterpriseModules::Oracle::SourceFunctions
  include PuppetX::EnterpriseModules::Oracle::JavaSourceHandling

  # Description of the content property, explaining its purpose and usage.
  desc <<-DESC
  A property describing the content of the function.

  Because puppet uses the contents of this property to decide if something needs to be done, the
  contents of this file must be very strict to the way Oracle stores the function source in the database.

  To decide if a change must be done, Puppet will fetch the function definition from the database and the contents
  of the source. For both it will remove all:

  - comments
  - empty lines
  - spaces
  - newlines
  - quotes

  and do a comparison.
  DESC

  data_type('Optional[String[1]]')

  # Translates the raw resource to the resource's content in the database.
  # @param _raw_resource [Object] The raw resource to translate.
  # @return [String] The content in the database.
  to_translate_to_resource do |_raw_resource|
    'content-in-database'
  end

  # Retrieves the required content for the Java source function.
  # @return [String] The processed required content.
  def required_content
    @required_content ||= process_content(value)
  end
end
