# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'puppet_x/enterprisemodules/oracle/mungers'

# Defines a new parameter for the ora_java_source type.
newparam(:name) do
  include EasyType

  # @!attribute [r] name
  #   @return [String] The full Java name including a SID.
  #
  #   The format is expected to be 'owner.java@sid'. The SID is optional; if not provided,
  #   Puppet will use the database instance with the `default` attribute set to true
  #   in the `/etc/ora_setting.yaml` file.
  #
  #   @example
  #     ora_java_source { 'owner.java@sid': ... }
  desc <<-DESC
  The full java name including a SID.

      ora_java_source { 'owner.java@sid':
        ...
      }

  The SID is optional. When you don't specify an SID, Puppet will take the database instance which has
  the `default` attribute set to true in `/etc/ora_setting.yaml` file and use that as the SID. We
  recommend to **always** use a full qualified name (e.g. a name including the SID).
  DESC

  isnamevar
  data_type('String[1]')

  # Translates the raw resource data into a resource-friendly format.
  #
  # @param raw_resource [Puppet::Resource] The raw resource containing the data.
  # @return [String] The formatted Java name including the SID.
  to_translate_to_resource do |raw_resource|
    sid = raw_resource.column_data('SID')
    owner = raw_resource.column_data('OWNER').upcase
    java_name = raw_resource.column_data('OBJECT_NAME')
    "#{owner}.#{java_name}@#{sid}"
  end
end
