# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#

# This class defines a Puppet property for managing Oracle Java source definitions.
# It includes methods for comparing the current and required content of the Java source.
newproperty(:source) do
  include EasyType
  include PuppetX::EnterpriseModules::Oracle::Access
  include PuppetX::EnterpriseModules::Oracle::SourceFunctions
  include PuppetX::EnterpriseModules::Oracle::JavaSourceHandling

  desc <<-DESC
  A file describing the content of the java source.

  Because puppet uses the contents of this file to decide if something needs to be done, the
  contents of this file must be very strict to the way Oracle stores the java source source in the database.

  To decide if a change must be done, Puppet will fetch the java source definition from the database and the contents
  of the source. For both it will remove all:

  - comments
  - empty lines
  - spaces
  - newlines
  - quotes

  and do a comparison.

  DESC

  data_type('Optional[String[1]]')

  # Translates the raw resource to a string representation for the resource.
  #
  # @param _raw_resource [Object] The raw resource to be translated.
  # @return [String] A string representation of the resource.
  to_translate_to_resource do |_raw_resource|
    'content-in-database'
  end

  # Reads the required content from the specified file.
  #
  # @return [String] The required content read from the file.
  def required_content
    @required_content ||= process_content(File.read(value))
  end
end
