# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
# Docs
module PuppetX
  module EnterpriseModules
    module Oracle
      # This module handles Oracle function definitions and their processing.
      #
      # @module FunctionHandling
      #
      # Provides methods to manage and manipulate Oracle function definitions
      # within Puppet configurations.
      module FunctionHandling
        # Converts the current value of a function definition to a string.
        #
        # @param currentvalue [String] The current value of the function definition.
        # @param _newvalue [String] The new value (not used in this method).
        # @return [String] A message indicating the status of the function definition.
        def change_to_s(currentvalue, _newvalue)
          if currentvalue.to_s == 'absent'
            'created function definition'
          else
            'changed to new function definition'
          end
        end

        # Retrieves the current content of the function definition.
        #
        # @return [String] The processed content of the function definition.
        def current_content
          @current_content ||= process_content(sql(template('ora_config/ora_function/function_definition.sql.erb', binding), :sid => resource.sid, :parse => false))
        end

        # Processes the content of the function definition by removing unnecessary
        # characters and formatting it.
        #
        # @param content [String] The raw content of the function definition.
        # @return [String] The cleaned and formatted content.
        def process_content(content)
          new_content = content.gsub(/CONTENT\s*\n-*\s*\n/, '').
                        gsub(/--.*\n/, '').
                        gsub(%r{^/\s*$}, '').
                        gsub(/\s|\s|"|'/, '').
                        delete("\n").upcase
          if resource[:editionable].to_s == 'false'
            new_content.gsub(/(?:non)?editionable/i, '')
          else
            new_content
          end
        end

        # Checks for compilation errors in the function definition.
        #
        # @return [Integer] The number of compilation errors found.
        def compilation_errors
          results = sql template('ora_config/ora_function/function_errors.sql.erb', binding), :sid => resource.sid
          results.first['ERRORS'].to_i
        end
      end
    end
  end
end
