# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
# Docs

module PuppetX
  module EnterpriseModules
    module Oracle
      # The JavaSourceHandling module provides methods to manage and process Java source definitions
      # within an Oracle database. It includes functionality to retrieve, process, and describe changes
      # to Java source definitions, as well as to check for compilation errors.
      #
      # This module is designed to be used within Puppet manifests to ensure that Java source definitions
      # are correctly managed and maintained in the Oracle database.
      #
      module JavaSourceHandling
        # Retrieves the current content of the Java source definition from the database.
        #
        # @return [String] The current content of the Java source definition.
        def current_content
          @current_content ||= process_content(sql(template('ora_config/ora_java_source/java_source_definition.sql.erb', binding), :sid => resource.sid, :parse => false))
        end

        # Returns a string representation of the change based on the current value.
        #
        # @param currentvalue [String] The current value of the resource.
        # @param _newvalue [String] The new value to be set.
        # @return [String] A description of the change made to the resource.
        def change_to_s(currentvalue, _newvalue)
          if currentvalue.to_s == 'absent'
            'created java source definition'
          else
            'changed to new java source definition'
          end
        end

        # Processes the content by removing unnecessary elements and formatting it for comparison.
        #
        # @param content [String] The content to be processed.
        # @return [String] The processed content, formatted for comparison.
        def process_content(content)
          content.gsub(/CREATE(\s|\n)+OR(\s|\n)+REPLACE(\s|\n)+AND(\s|\n)+RESOLVE(\s|\n)+JAVA(\s|\n)+SOURCE(\s|\n)+NAMED.*AS(\s|\n)+/mi, '').
            gsub(/1\* SELECT text as content FROM all_source WHERE name.*/, '').
            gsub(/--.*\n/, '').
            gsub(%r{^/\s*$}, '').
            gsub(/\s|\s|"|'/, '').
            delete("\n").upcase
        end

        def compilation_errors
          results = sql template('ora_config/ora_java_source/java_source_errors.sql.erb', binding), :sid => resource.sid
          results.first['ERRORS'].to_i
        end
      end
    end
  end
end
