# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
# Docs
module PuppetX
  module EnterpriseModules
    module Oracle
      # Module for handling Oracle procedure definitions.
      #
      # This module provides methods to manage and process Oracle
      # procedure definitions within Puppet.
      module ProcedureHandling
        # Converts the current value to a string representation.
        #
        # @param currentvalue [Object] The current value of the procedure.
        # @param _newvalue [Object] The new value (not used).
        # @return [String] A message indicating the status of the procedure.
        def change_to_s(currentvalue, _newvalue)
          if currentvalue.to_s == 'absent'
            'created procedure definition'
          else
            'changed to new procedure definition'
          end
        end

        # Retrieves the current content of the procedure definition.
        #
        # @return [String] The processed content of the procedure definition.
        def current_content
          @current_content ||= process_content(sql(template('ora_config/ora_procedure/procedure_definition.sql.erb', binding), :sid => resource.sid, :parse => false))
        end

        # Processes the content of the procedure definition.
        #
        # @param content [String] The raw content of the procedure definition.
        # @return [String] The cleaned and formatted content.
        def process_content(content)
          new_content = content.gsub(/CONTENT\s*\n-*\s*\n/, '').
                        gsub(/--.*\n/, '').
                        gsub(%r{^/\s*$}, '').
                        gsub(/\s|\s|"|'/, '').
                        delete("\n").upcase
          if resource[:editionable].to_s == 'false'
            new_content.gsub(/(?:non)?editionable/i, '')
          else
            new_content
          end
        end

        # Retrieves the compilation errors from the procedure.
        #
        # @return [Integer] The number of compilation errors found.
        def compilation_errors
          results = sql template('ora_config/ora_procedure/procedure_errors.sql.erb', binding), :sid => resource.sid
          results.first['ERRORS'].to_i
        end
      end
    end
  end
end
