# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
# Docs
module PuppetX
  module EnterpriseModules
    module Oracle
      # This module provides functions to manage and compare Oracle source content.
      #
      # It includes methods to check for errors, determine if the current content is in sync
      # with the required content, and show differences between the two.
      #
      # @example Usage
      #   if source_functions.insync?(current_state)
      #     # Do something if in sync
      #   end
      module SourceFunctions
        # Checks if there are any compilation errors.
        #
        # @return [Boolean] True if there are compilation errors, false otherwise.
        def contains_errors?
          compilation_errors > 0
        end

        # Compares the current content with the required content to determine if they are in sync.
        #
        # @param _is [Object] The current state of the resource.
        # @return [Boolean] True if the current content matches the required content, false otherwise.
        #
        # @example
        #   if insync?(current_state)
        #     # Content is in sync
        #   end
        def insync?(_is)
          if current_content == required_content
            Puppet.warning "#{path}: #{resource.owner}.#{resource.java_name} up-to-date, but contains #{compilation_errors} error(s)." \
              if contains_errors? && resource.report_errors
            return true
          end
          show_diff(current_content, required_content)
          false
        end

        # Shows the difference between current and required content.
        #
        # @param current_content [String] The current content to compare.
        # @param required_content [String] The required content to compare.
        def show_diff(current_content, required_content)
          # Use a single Tempfile for both contents to reduce overhead
          Tempfile.create(['ora_java_source', nil]) do |current_content_file|
            current_content_file.write(current_content)
            current_content_file.flush

            Tempfile.create(['ora_java_source', nil]) do |required_content_file|
              required_content_file.write(required_content)
              required_content_file.flush

              file_diff = `diff #{current_content_file.path} #{required_content_file.path}`
              Puppet.info "#{path}: Difference is:\n#{file_diff}"
            end
          end
        end
      end
    end
  end
end
